/**
 * @file IMU.c
 * @author Jonas Gelhaus (jonas.gelhaus@byteways.de)
 * @brief 
 * @version 0.1
 * @date 2022-06-29
 * 
 * @copyright Copyright (c) 2022
 * 
 */

#include <inttypes.h>
#include <drivers/sensor.h>
#include "IMU.h"
#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>

bool active = true;
bool powered = true;

static inline float out_ev(struct sensor_value *val)
{
	return (val->val1 + (float)val->val2 / 1000000);
}


uint16_t get_rotation(const struct device *dev)
{
	struct sensor_value value[3];
	static int trig_cnt;
	double irpm;

	trig_cnt++;

	/* lsm6dso gyro */
	sensor_sample_fetch_chan(dev, SENSOR_CHAN_GYRO_XYZ);
	sensor_channel_get(dev, SENSOR_CHAN_GYRO_XYZ, &value);

	/* save data in double variable */
	double xg = out_ev(&value[0])*1000.0;
	double yg = out_ev(&value[1])*1000.0;

	printk("\nXG-Wert: %f\n",xg );
	printk("YG-Wert: %f\n",yg );

	/* get absolut from rotation value */
	double xrpm =  abs(xg) > abs(yg) ? abs(xg) : abs(yg);
	
	irpm = (xrpm * 60.0 / (65536.0 / 4000.0) / 360.0); // TODO precision/overflow check
	printk("%f rpm\n", irpm);
	return irpm;
}

int set_sampling_freq(const struct device *dev)
{
	int ret = 0;
	struct sensor_value odr_attr;

	/* set accel/gyro sampling frequency to 12.5 Hz */
	odr_attr.val1 = 12.5;
	odr_attr.val2 = 0;

	ret = sensor_attr_set(dev, SENSOR_CHAN_ACCEL_XYZ,
			SENSOR_ATTR_SAMPLING_FREQUENCY, &odr_attr);
	if (ret != 0) {
		printf("Cannot set sampling frequency for accelerometer.\n");
		return ret;
	}

	ret = sensor_attr_set(dev, SENSOR_CHAN_GYRO_XYZ,
			SENSOR_ATTR_SAMPLING_FREQUENCY, &odr_attr);
	if (ret != 0) {
		printf("Cannot set sampling frequency for gyro.\n");
		return ret;
	}
	return 0;
}